{-----------------------------------------------------------------------------
 Unit Name: Engine
 Author:    Nitrogen
 Purpose:   Provides the Audio/visual interface between the viewscreen and the octtree stored level.
 History:   21 July - Created.
-----------------------------------------------------------------------------}


unit Engine;


interface

Uses Windows, OpenGl12, TGATexture, Globals, Octtree, FrustumClass, Skybox;

Procedure engine_init;
Procedure engine_loop;
Procedure engine_deinit;

Var
   Scene: Array[0..1] of HybdFile;
   CurScene: integer = 0;
   Loading: boolean = true;
   LoadingDone: integer = 0;
implementation

Uses Log, Particles, Water;

Var Frustum:  TFrustum;
    FlareTex: array[0..10] of gluint;
    LoadTex, FadeTex: gluint;

    Pnt: array[0..1] of pointlist;
    Smoke: array[0..3] of ParticleSystem;
    Fire: array[0..7] of ParticleSystem;
    P: vector;

    CurTex, CurLtex: integer;

    Pathind: integer;
    Pathinc: glFloat;
    PathStart: integer;
    DoneScene, BlendEnabled: boolean;
    Diff: single;

    Sx, Sy, Sz: glDouble; //Sun Position...
    flareCol, FlareSize: glFloat;

    Hw, Hh: glfloat;
    LastTime, CurTime, Freq: int64;

Procedure CreateOctDL(O: POctNode; ind: integer); forward;


Procedure InitParticles;
Var I, J: integer;
begin
    Pnt[0].Load('Data\First.pnt');
    Pnt[1].Load('Data\Main.pnt');

if Curscene = 0 then
begin

   LoadTGA('Data\Smoke.tga'     , Smoke[0].Tex, True);
   LoadTGA('Data\SmokeBlack.tga', Smoke[1].Tex, True);

  for I := 0 to 3 do
  begin
    Smoke[I].Init(
    Pnt[0].P[I].P[0],
    Pnt[0].P[I].P[1],
    Pnt[0].P[I].P[2],

    Pnt[0].P[I].M[0]/60,
    Pnt[0].P[I].M[1]/60,
    Pnt[0].P[I].M[2]/60,
    0,14);
    Smoke[I].App := 0.04;
    Smoke[I].Apv := 0.04;
    smoke[I].Damp :=0.98;
    smoke[I].Lif := 300;
    if I <= 1 then
    Smoke[I].Size := 3
    else
    Smoke[I].Size := 2;

    if I <= 1 then
    smoke[I].MaxAni := 9
    else
    smoke[I].MaxAni := 12;

    Smoke[I].Color := 0.2;
    Smoke[I].Reset;
  end;

end
else
begin
   LoadTGA('Data\Fire.tga'     , Fire[0].Tex, True);
   LoadTGA('Data\Water.tga'     , Fire[1].Tex, True);

  for I := 0 to 1 do
  begin
    Fire[I].Init(
    Pnt[1].P[I].P[0],
    Pnt[1].P[I].P[1],
    Pnt[1].P[I].P[2],

    Pnt[1].P[I].M[0]/60,
    Pnt[1].P[I].M[1]/60,
    Pnt[1].P[I].M[2]/60,
    0,20);
    Fire[I].Mode := 1;

    Fire[I].App := 8;
    Fire[I].Apv := 0.08;
    Fire[I].Damp :=0.93;
    Fire[I].Lif := 100;
    Fire[I].Size := 13;
    fire[I].Grav := 0.005;
    Fire[I].MaxAni := 13;

    Fire[I].Color := 1;
    Fire[I].Reset;
  end;

  for I := 4 to 6 do
  begin
    Fire[I].Init(
    Pnt[1].P[I].P[0],
    Pnt[1].P[I].P[1],
    Pnt[1].P[I].P[2],

    Pnt[1].P[I].M[0]/20,
    Pnt[1].P[I].M[1]/20,
    Pnt[1].P[I].M[2]/20,
    0,200);
    Fire[I].Mode := 0;

    Fire[I].App := 0.08;
    Fire[I].Apv := 0.08;
    Fire[I].Damp :=1;
    Fire[I].Lif := 200;
    Fire[I].Size := 0.2;

    for J := 0 to High(Fire[I].P) do
    Fire[I].P[J].Line := true;

    fire[I].Grav := -0.005;
    Fire[I].MaxAni := 1;

    Fire[I].Color := 1;
    Fire[I].Reset;
  end;


  for I := 2 to 3 do
  begin
    Fire[I].Init(
    Pnt[1].P[I].P[0],
    Pnt[1].P[I].P[1],
    Pnt[1].P[I].P[2],

    Pnt[1].P[I].M[0]/60,
    Pnt[1].P[I].M[1]/60,
    Pnt[1].P[I].M[2]/60,
    0,10);
    Fire[I].Mode := 1;

    Fire[I].App := 8;
    Fire[I].Apv := 0.08;
    Fire[I].Damp :=0.93;
    Fire[I].Lif := 100;
    Fire[I].Size := 13;
    fire[I].Grav := 0.005;
    Fire[I].MaxAni := 13;

    Fire[I].Color := 1;
    Fire[I].Reset;
  end;



end;

   glAlphaFunc(GL_GREATER, 0.01);
end;

Procedure engine_init;
Var I: integer;
Begin
  addtolog('Engine Init');
  glDisable(GL_LIGHTING);
  glenable(GL_DEPTH_TEST);
  glDisable(GL_BLEND);


  QueryPerformanceFrequency(Freq);
  if freq = 0 then
  Addtolog('Performance counter not installed!, Scrolling bits wont work.');


  glActiveTextureARB(GL_TEXTURE0_ARB);
  glEnable(GL_TEXTURE_2D);
 // glTexGenf(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
 // glTexGenf(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);

  glActiveTextureARB(GL_TEXTURE1_ARB);
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_CULL_FACE);
  DoneScene := False;

  glActiveTextureARB(GL_TEXTURE0_ARB);

  InitSkybox;
  FlareSize := Startupinfo.Wid/4;
  Hw := Startupinfo.Wid/2;
  HH := Startupinfo.Hgt/2;

  Sun[0] := -176;
  Sun[1] := -20;
  Sun[2] := 253;

  Particles_Init;
  //InitParticles;

  CreateSkybox(200);

  Loading := true;
  Loadingdone := 0;
  LoadTGA('Data\Black.tga', FadeTex, False);
  LoadJPG('Data\Loading.jpg', LoadTex);

  for I := 0 to 8 do
  LoadJPG('Data\Flare'+inttostr(I)+'.jpg', FlareTex[I]);

  glClear(GL_COLOR_BUFFER_BIT);
  SwitchView(False);
  curScene := 0;



end;


Procedure DoPath(ind: integer);
Var aa, bb, bb2, bb3, aa2, aa3: single;
    V: vector;
begin
   PathInc := (ElapsedTime - PathStart) / Scene[ind].C[Pathind].Tim;
   if pathInc > 1 then
   begin
      inc(Pathind);
      PathInc := 0;
      PathStart := ElapsedTime;
      if Pathind > high(Scene[ind].c) then
      begin
        DoneScene := true;
        exit;
      end;
   end;


       aa := PathInc;
       bb := 1- PathInc;
       bb2 := bb*bb;
       bb3 := bb*bb*bb;
       aa2 := aa*aa;
       aa3 := aa*aa*aa;

       P[0] := Scene[ind].C[Pathind].L[0].P[0][0]*bb3 + 3*Scene[ind].C[Pathind].L[0].C[0][0]*bb2*aa + 3*Scene[ind].C[Pathind].L[1].C[0][0]*bb*aa2 + Scene[ind].C[Pathind].L[1].P[0][0]*aa3;
       P[1] := Scene[ind].C[Pathind].L[0].P[0][1]*bb3 + 3*Scene[ind].C[Pathind].L[0].C[0][1]*bb2*aa + 3*Scene[ind].C[Pathind].L[1].C[0][1]*bb*aa2 + Scene[ind].C[Pathind].L[1].P[0][1]*aa3;
       P[2] := Scene[ind].C[Pathind].L[0].P[0][2]*bb3 + 3*Scene[ind].C[Pathind].L[0].C[0][2]*bb2*aa + 3*Scene[ind].C[Pathind].L[1].C[0][2]*bb*aa2 + Scene[ind].C[Pathind].L[1].P[0][2]*aa3;

       V[0] := Scene[ind].C[Pathind].L[0].P[1][0]*bb3 + 3*Scene[ind].C[Pathind].L[0].C[1][0]*bb2*aa + 3*Scene[ind].C[Pathind].L[1].C[1][0]*bb*aa2 + Scene[ind].C[Pathind].L[1].P[1][0]*aa3;
       V[1] := Scene[ind].C[Pathind].L[0].P[1][1]*bb3 + 3*Scene[ind].C[Pathind].L[0].C[1][1]*bb2*aa + 3*Scene[ind].C[Pathind].L[1].C[1][1]*bb*aa2 + Scene[ind].C[Pathind].L[1].P[1][1]*aa3;
       V[2] := Scene[ind].C[Pathind].L[0].P[1][2]*bb3 + 3*Scene[ind].C[Pathind].L[0].C[1][2]*bb2*aa + 3*Scene[ind].C[Pathind].L[1].C[1][2]*bb*aa2 + Scene[ind].C[Pathind].L[1].P[1][2]*aa3;



     gluLookat(P[0], P[1], P[2], V[0], V[1], V[2], 0, 0, 1);
end;

Procedure CalcOctVis(O: pOctNode);
Var I: integer;
begin
    if Frustum.CubeInFrustum(O^.P[0],O^.P[1], O^.P[2], O^.Wid) then
    begin
     O.Vis := true;
      for I := 0 to 7 do
       if O^.N[I] <> nil then
        CalcOctVis(O^.N[I]);
    end
    else
    O.Vis := false;


end;


Procedure CreateOctDL(O: POctNode; ind: integer);
Var I: integer;
    J, K, Tid, Lid: integer;
begin

  if O^.Fcount > 0 then
  begin

  O^.DL := glGenLists(1);
  glNewList(O^.DL, GL_COMPILE);


  For K := 0 to High(O^.F) do
  begin
    I := O^.F[K];
    Tid := scene[ind].F[I].Tid;
    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, scene[ind].Tex[Tid]);

    if scene[ind].F[I].Trans then
    begin
      if Not BlendEnabled then
      begin
       BlendEnabled := true;
       glEnable(GL_BLEND);
      end;
    end
    else
    begin
      if BlendEnabled then
      begin
       BlendEnabled := False;
       glDisable(GL_BLEND);
      end;
    end;

     Lid := scene[ind].F[I].Lid;
     CurLTex := Lid;
     glActiveTextureARB(GL_TEXTURE1_ARB);
     glBindTexture(GL_TEXTURE_2D, Scene[ind].LMTex[Lid]);

    glBegin(GL_TRIANGLES);
        for J := 0 to 2 do
        begin
          glMultiTexCoord2fARB(GL_TEXTURE0_ARB, scene[ind].F[I].T[J][0]+scene[ind].Sc[I].Uf,scene[ind].F[I].T[J][1]+scene[ind].Sc[I].Vf);
          glMultiTexCoord2fARB(GL_TEXTURE1_ARB, scene[ind].F[I].L[J][0],scene[ind].F[I].L[J][1]);

          glVertex3fv(@scene[ind].V[scene[ind].F[I].V[J]]);
        end;
    glEnd;

  end;

  glEndList;

  End
  else //Fcount > 0
  for I := 0 to 7 do
  if O^.N[I] <> nil then
  CreateOctDL(O^.N[I], ind);
end;


Procedure DrawOctScene(const O: pOctNode; const ind: integer);
Var I: integer;
     J, K, Tid, Lid: integer;
begin
  if O^.Vis = false then exit;
  if O^.Fcount > 0 then
  begin

  For K := 0 to High(O^.F) do
  begin
    I := O^.F[K];
    if scene[Ind].F[I].UScroll <> 0 then
    begin
    scene[ind].sc[I].Uf := scene[ind].sc[I].Uf + Diff*scene[ind].F[I].UScroll;
    end;

    if scene[ind].F[I].VScroll <> 0 then
    begin
    scene[ind].sc[I].Vf := scene[ind].sc[I].Vf + diff*scene[ind].F[I].VScroll;
    end;

    Tid := scene[ind].F[I].Tid;
   // if Tid <> CurTex then
    begin
     glActiveTextureARB(GL_TEXTURE0_ARB);
     glBindTexture(GL_TEXTURE_2D, scene[ind].Tex[Tid]);
   //  CurTex := Tid;
    end;
    if scene[ind].F[I].Trans then
    begin
      if Not BlendEnabled then
      begin
       BlendEnabled := true;
       glEnable(GL_BLEND);
      end;
    end
    else
    begin
      if BlendEnabled then
      begin
       BlendEnabled := False;
       glDisable(GL_BLEND);
      end;
    end;

    Lid := scene[ind].F[I].Lid;
  //  if Lid <> CurLTex then
    begin
     CurLTex := Lid;
     glActiveTextureARB(GL_TEXTURE1_ARB);
     glBindTexture(GL_TEXTURE_2D, Scene[ind].LMTex[Lid]);
    end;

    glBegin(GL_TRIANGLES);
        for J := 0 to 2 do
        begin
          glMultiTexCoord2fARB(GL_TEXTURE0_ARB, scene[ind].F[I].T[J][0]+scene[ind].Sc[I].Uf,scene[ind].F[I].T[J][1]+scene[ind].Sc[I].Vf);
          glMultiTexCoord2fARB(GL_TEXTURE1_ARB, scene[ind].F[I].L[J][0],scene[ind].F[I].L[J][1]);

          glVertex3fv(@scene[ind].V[scene[ind].F[I].V[J]]);
        end;
    glEnd;

  end;

  End
  else //Fcount > 0
  for I := 0 to 7 do
  if O.N[I] <> nil then
  DrawOctScene(O.N[I], ind);
end;


Procedure DrawOctDL(const O: pOctNode; const ind: integer);
Var I: integer;
begin
  if O^.Vis = false then exit;
  if O^.Fcount > 0 then
  begin
    glCallList(O^.DL);
  End
  else
  for I := 0 to 7 do
  if O.N[I] <> nil then
  DrawOctDL(O.N[I], ind);
end;


Procedure DrawScene(ind: integer);
Var I, J, Tid, Lid: integer;
begin
  Diff := (int64(CurTime)-int64(LastTime))/int64(freq)*100;

  For I := 0 to High(scene[ind].F) do
  if Scene[ind].Vis[I][pathind] = true then
  begin

    if scene[Ind].F[I].UScroll <> 0 then
    begin
    scene[ind].sc[I].Uf := scene[ind].sc[I].Uf + Diff*scene[ind].F[I].UScroll;
    end;

    if scene[ind].F[I].VScroll <> 0 then
    begin
    scene[ind].sc[I].Vf := scene[ind].sc[I].Vf + diff*scene[ind].F[I].VScroll;
    end;

    Tid := scene[ind].F[I].Tid;
    if Tid <> CurTex then
    begin
     glActiveTextureARB(GL_TEXTURE0_ARB);
     glBindTexture(GL_TEXTURE_2D, scene[ind].Tex[Tid]);
     CurTex := Tid;
    end;
    if scene[ind].F[I].Trans then
    begin
      if Not BlendEnabled then
      begin
       BlendEnabled := true;
       glEnable(GL_BLEND);
      end;
    end
    else
    begin
      if BlendEnabled then
      begin
       BlendEnabled := False;
       glDisable(GL_BLEND);
      end;
    end;

    Lid := scene[ind].F[I].Lid;
    if Lid <> CurLTex then
    begin
     CurLTex := Lid;
     glActiveTextureARB(GL_TEXTURE1_ARB);
     glBindTexture(GL_TEXTURE_2D, Scene[ind].LMTex[Lid]);
    end;

    glBegin(GL_TRIANGLES);
        for J := 0 to 2 do
        begin
          glMultiTexCoord2fARB(GL_TEXTURE0_ARB, scene[ind].F[I].T[J][0]+scene[ind].Sc[I].Uf,scene[ind].F[I].T[J][1]+scene[ind].Sc[I].Vf);
          glMultiTexCoord2fARB(GL_TEXTURE1_ARB, scene[ind].F[I].L[J][0],scene[ind].F[I].L[J][1]);

         // glNormal3fv(@N[scene[ind].F[I].V[J]]);
          glVertex3fv(@scene[ind].V[scene[ind].F[I].V[J]]);
        end;
    glEnd;

  end;


end;

Procedure DrawQuad(Size: glFloat);
begin
       glbegin(GL_QUADS);
          glTexCoord2f(0,0);glVertex3f(-size,-size,0);
          glTexCoord2f(1,0);glVertex3f( size,-size,0);
          glTexCoord2f(1,1);glVertex3f( size, size,0);
          glTexCoord2f(0,1);glVertex3f(-size, size,0);
       glend;
end;

Procedure DoFlare;
Var Col: boolean;
   FlX, FlY: glFloat;
begin
   GetFlarePos(Sx, Sy, Sz, Col);
   if col then
   if Flarecol < 0.7 then Flarecol := FlareCol + 0.1;
   if not col then
   if Flarecol > 0 then Flarecol := FlareCol - 0.1;

   Flx := Sx-hw;
   Fly := Sy-hh;
   if FlareCol > 0 then
   begin
       glDisable(GL_DEPTH_TEST);
       glDisable(GL_CULL_FACE);
       glEnable(GL_BLEND);
       glBlendFunc(GL_ONE, GL_ONE);


       glActiveTextureARB(GL_TEXTURE1_ARB);
       glDisable(GL_TEXTURE_2D);
       glActiveTextureARB(GL_TEXTURE0_ARB);


       glcolor3f(FlareCol,FlareCol,FlareCol);

       switchView(true);
       glPushMatrix;
        glTranslatef(hw+Flx, hh+fly, -5);
        glRotatef(Sx/hw*90, 0, 0, 1);
        glBindTexture(GL_TEXTURE_2D, FlareTex[0]);
        DrawQuad(FlareSize);
       glPopMatrix;
       glPushMatrix;
        glTranslatef(hw+Flx, hh+fly, -5);
        glBindTexture(GL_TEXTURE_2D, FlareTex[1]);
        DrawQuad(FlareSize*1.1);
       glPopMatrix;

       glColor3f(FlareCol-0.1,FlareCol-0.1,FlareCol-0.1);
       glPushMatrix;
        glTranslatef(hw-Flx, hh-fly, -5);
        glBindTexture(GL_TEXTURE_2D, FlareTex[2]);
        DrawQuad(FlareSize*0.8);
       glPopMatrix;
       glPushMatrix;
        glTranslatef(hw-Flx*0.9, hh-fly*0.9, -5);
        glBindTexture(GL_TEXTURE_2D, FlareTex[3]);
        DrawQuad(FlareSize*0.4);
       glPopMatrix;
       glPushMatrix;
        glTranslatef(hw+Flx*1.5, hh+fly*1.5, -5);
        glBindTexture(GL_TEXTURE_2D, FlareTex[8]);
        DrawQuad(FlareSize*0.3);
       glPopMatrix;
       glPushMatrix;
        glTranslatef(hw-Flx*0.65, hh-fly*0.65, -5);
        glBindTexture(GL_TEXTURE_2D, FlareTex[7]);
        DrawQuad(FlareSize*0.2);
       glPopMatrix;

       glPushMatrix;
      glTranslatef(hw-Flx*0.35, hh-fly*0.35, -5);
        glBindTexture(GL_TEXTURE_2D, FlareTex[6]);
        DrawQuad(FlareSize*0.15);
       glPopMatrix;
       glPushMatrix;
        glTranslatef(hw-Flx*0.1, hh-fly*0.1, -5);
        glBindTexture(GL_TEXTURE_2D, FlareTex[4]);
        DrawQuad(FlareSize*0.05);
       glPopMatrix;
       glPushMatrix;
        glTranslatef(hw-Flx*0.3, hh-fly*0.3, -5);
        glBindTexture(GL_TEXTURE_2D, FlareTex[5]);
        DrawQuad(FlareSize*0.3);
       glPopMatrix;
       glPushMatrix;
        glTranslatef(hw+Flx*0.5, hh+fly*0.5, -5);
        glBindTexture(GL_TEXTURE_2D, FlareTex[7]);
        DrawQuad(FlareSize*0.2);
       glPopMatrix;

       glDisable(GL_BLEND);
       glActiveTextureARB(GL_TEXTURE1_ARB);
       glEnable(GL_TEXTURE_2D);
       glActiveTextureARB(GL_TEXTURE0_ARB);
       glEnable(GL_DEPTH_TEST);
       glEnable(GL_CULL_FACE);

       switchview(false);
   end;



end;

function CamIn(A,B: integer): boolean;
begin
    result := False;
    if (Pathind >= A) and (Pathind <= B) then
    Result := true;

end;

Procedure DoParticles;
Var I: integer;
begin
  Particles_GetMat;
  glActiveTextureARB(GL_TEXTURE1_ARB);
  glDisable(GL_TEXTURE_2D);

  glActiveTextureARB(GL_TEXTURE0_ARB);

  glEnable(GL_BLEND);
  glDisable(GL_CULL_FACE);
  glEnable(GL_ALPHA_TEST);
  //glDisable(GL_DEPTH_TEST);

if curscene = 0 then
begin
  glBindTexture(GL_TEXTURE_2D, Smoke[0].tex);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE);

   Smoke[0].Tick;
   Smoke[0].Draw;

   Smoke[1].Tick;
   Smoke[1].Draw;

  glBindTexture(GL_TEXTURE_2D, Smoke[1].tex);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

   Smoke[2].Tick;
   Smoke[2].Draw;
   Smoke[3].Tick;
   Smoke[3].Draw;

end;


if curscene = 1 then
begin
  glBindTexture(GL_TEXTURE_2D, Fire[0].tex);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE);

  for I :=  0 to 1 do
  begin
   Fire[I].Tick;
   Fire[I].Draw;
  end;

  glBlendFunc(GL_SRC_COLOR, GL_ONE);

  for I :=  2 to 3 do
  begin
   Fire[I].Tick;
   Fire[I].Draw;
  end;

  glBindTexture(GL_TEXTURE_2D, Fire[1].tex);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE);

  for I :=  4 to 7 do
  begin
   Fire[I].Tick;
   Fire[I].Draw;
  end;


end;


  glDisable(GL_ALPHA_TEST);
  glDisable(GL_BLEND);
  glEnable(GL_CULL_FACE);
  glBlendFunc(GL_ONE, GL_ONE);
  glActiveTextureARB(GL_TEXTURE1_ARB);
  glEnable(GL_TEXTURE_2D);
  glColor3f(1,1,1);
end;

Procedure engine_Loop;
Var Col, Hw, Hh: glFloat;
    Diff: glFloat;
begin
  LastTime := CurTime;
  QueryPerformanceCounter(CurTime);


 if Loading then
 begin
    if LoadingDone < 3 then
    begin
     Switchview(True);
     glDisable(GL_CULL_FACE);
     glActiveTextureARB(GL_TEXTURE1_ARB);
     glDisable(GL_TEXTURE_2D);

     glActiveTextureARB(GL_TEXTURE0_ARB);
     glBindTexture(GL_TEXTURE_2D, LoadTex);
     glColor3f(1,1,1);
     glTranslatef(0,0,-15);
     Hw := Startupinfo.Wid/2;
     Hh := Startupinfo.Hgt/2;

     glBegin(GL_QUADS);
        glTexCoord2f(0,0);glVertex3f(Hw-128, Hh-32, 0);
        glTexCoord2f(1,0);glVertex3f(Hw+128, Hh-32, 0);
        glTexCoord2f(1,1);glVertex3f(Hw+128, Hh+32, 0);
        glTexCoord2f(0,1);glVertex3f(Hw-128, Hh+32, 0);
     glEnd;
     inc(Loadingdone);
     Switchview(false);
     glEnable(GL_CULL_FACE);
     if LoadingDone = 2 then
    begin
     glActiveTextureARB(GL_TEXTURE0_ARB);
     glEnable(GL_TEXTURE_2D);

      if Curscene = 0 then
       LoadHybd('Data\First.hybd', Scene[Curscene])
       else
       begin
       UnloadHybdTextures(Scene[0]);
       LoadHybd('Data\Main.hybd', Scene[Curscene]);
       end;

       LoadHybdTextures(Scene[Curscene]);
       InitParticles;
       Water_Init;
       
     glActiveTextureARB(GL_TEXTURE1_ARB);
     glEnable(GL_TEXTURE_2D);

    end;
    end
    else
    begin

       Loading := False;
       LoadingDone := 0;
       PathStart := ElapsedTime;
       PathInd := 0;
       PathInc := 0;

       if Curscene = 0 then
         Music.SetOrder(10)
       else
         Music.SetOrder(21);


    end;

 end
 else
 begin

   DoPath(CurScene);
 if Curscene = 0 then
 if Pathind < 10 then
 begin
   glPushMatrix;
   glActiveTextureARB(GL_TEXTURE1_ARB);
   gldisable(GL_TEXTURE_2D);

   glActiveTextureARB(GL_TEXTURE0_ARB);
   glDisable(GL_DEPTH_TEST);
     glColor3f(1,1,1);
     glRotatef(90, 1, 0, 0);
     glCallList(SkyboxDL);
   glEnable(GL_DEPTH_TEST);
   glClear(GL_DEPTH_BUFFER_BIT);

   glActiveTextureARB(GL_TEXTURE1_ARB);
   glEnable(GL_TEXTURE_2D);
   glPopMatrix;
 end;
   //MAIN PROCEDURE


   DrawScene(CurScene);


 DoParticles;


 if Curscene = 1 then
 begin
   glPushMatrix;
   //glLoadIdentity;
   //glrotatef(90, 1, 0,0);
   gltranslatef(673.199, -416.199, -103.199);

   //glEnable(GL_BLEND);
   glBlendFunc(GL_ONE, GL_ONE);
   glDisable(GL_ALPHA_TEST);
   glDisable(GL_CULL_FACE);

   glScalef(20,20,20);
   Water_Draw;

   glEnable(GL_CULL_FACE);
   glDisable(GL_BLEND);
   glPopMatrix;
 end;

 if Curscene = 0 then
 if Pathind < 10 then
 begin
 glPushMatrix;
  Doflare;
 glPopMatrix;
 end;


  if Scene[curscene].C[Pathind].fadein or Scene[curscene].C[Pathind].fadeout then
  begin
    Col := -1;
   if Elapsedtime-PathStart <= Scene[curscene].C[pathind].Tim div 2 then
   begin
    if Scene[curscene].C[Pathind].Fadein then
    Col := 1-((ElapsedTime-PathStart)/(Scene[curscene].C[pathind].FadeinTime));
   end
   else
    if Scene[curscene].C[Pathind].Fadeout then
    begin
    Col := (ElapsedTime-PathStart-(Scene[curscene].C[Pathind].Tim-Scene[curscene].C[Pathind].FadeoutTime))/(Scene[curscene].C[Pathind].FadeoutTime);
    end;
    if (Col <= 1) and (col >= 0) then
    begin
     glActiveTextureARB(GL_TEXTURE0_ARB);
     glBindTexture(GL_TEXTURE_2D, Fadetex);
     gldisable(GL_DEPTH_TEST);
     glEnable(GL_BLEND);
     glDisable(GL_CULL_FACE);

     Switchview(true);

     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

     glColor4f(1,1,1,Col);

     glTranslatef(0,0,-15);
     glBegin(GL_QUADS);
      glTexCoord2f(0,0);glVertex3f(0,               Startupinfo.Hgt , 0);
      glTexCoord2f(1,0);glVertex3f(Startupinfo.Wid, Startupinfo.Hgt , 0);
      glTexCoord2f(1,1);glVertex3f(Startupinfo.Wid, 0, 0);
      glTexCoord2f(0,1);glVertex3f(0,               0, 0);
     glEnd;

     Switchview(false);

     glBlendFunc(GL_ONE, GL_ONE);
     glEnable(GL_DEPTH_TEST);
     glDisable(GL_BLEND);
     glEnable(GL_CULL_FACE);
   end;
  end;


   if DoneScene then
   begin
      if Curscene = 0 then
      begin
         Curscene := 1;
         Loading := true;
         LoadingDone := 0;
         DoneScene := false;
      end
      else
      begin
        doneScene := false;
        Stage := 3;
        StageChange := true;

      end;
   end;

 end;



end;

Procedure engine_deinit;
Var I: integer;
begin
    Addtolog('Engine De-init');
 if GL_VERSION_1_2 then
 begin
    glDeleteTextures(1, @FadeTex);
    glDeleteTextures(1, @LoadTex);
    For I := 0 to high(FlareTex) do
    glDeleteTextures(1, @flareTex[I]);

    UnLoadHybdTextures(Scene[curscene]);
 end;
end;




end.
