// -----------
// Global Unit
// -----------
// For Gamedev.net 4e3 Compo.
//
// by Michael Pote (Nitrogen)
// Created: 2 May

unit Globals;

interface

uses Windows, OpenGL12, Sound, Font;

Const
     SetupWIDTH = 500;
     SetupHEIGHT = 400;
     HSW = SetupWidth/2;
     HSH = SetupHeight/2;
     TexturePath = 'Data\Textures\';
     StageJump = 1;
     fmOpenRead       = $0000;
     fmOpenWrite      = $0001;


type
     StartInfo = record
                  Wid, Hgt: integer;
                  BPP, Res: integer;
                  Fullscreen: boolean;
                 end;

var
    h_Wnd  : HWND;                       // Global window handle
    h_DC   : HDC;                        // Global device context
    h_RC   : HGLRC;                      // OpenGL rendering context
    FPSCount : Integer = 0;              // Counter for FPS

    StartupInfo: StartInfo; //Holds current window specs..
    Stage: integer = -2;
    ElapsedTime, TimeOffset: longword;
    keys   : Array[0..255] of Boolean;   // Holds keystrokes
    Finished : Boolean;
    F: FontObj;

    Music: TrackedStream;

    StageChange: boolean;

    EXEPath: string;


procedure glGenTextures(n: GLsizei; var textures: GLuint); stdcall; external 'opengl32.dll';
procedure glBindTexture(target: GLenum; texture: GLuint); stdcall; external 'opengl32.dll';

function IntToStr(Num : Integer) : String;
Procedure SwitchView(Ortho: boolean);
Procedure InitEnvMapping;
Procedure EnableEnv;
Procedure DisableEnv;

implementation

Procedure InitEnvMapping;
begin
  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glTexGeni(GL_S, GL_SPHERE_MAP, 0);
  glTexGeni(GL_T, GL_SPHERE_MAP, 0);
end;

Procedure EnableEnv;
begin
  glEnable(GL_TEXTURE_GEN_S);
  glEnable(GL_TEXTURE_GEN_T);
end;

Procedure DisableEnv;
begin
  glDisable(GL_TEXTURE_GEN_S);
  glDisable(GL_TEXTURE_GEN_T);
end;



function IntToStr(Num : Integer) : String;
begin
  Str(Num, result);
end;

Procedure SwitchView(Ortho: boolean);
Var Size: integer;
begin
  //glClear(GL_COLOR_BUFFER_BIT);
  glMatrixMode(GL_PROJECTION);        // Change Matrix Mode to Projection
  glLoadIdentity();                   // Reset View
  if not Ortho then
  begin
   Size := Startupinfo.Hgt div 18;
   if Stage = 2 then
   begin
    glViewport(0, Size, Startupinfo.Wid, Startupinfo.Hgt-Size*2);
    gluPerspective(70.0, Startupinfo.Wid/(Startupinfo.Hgt-Size*2), 0.01, 1000.0)
   end
   else
   begin
    glViewport(0, 0, Startupinfo.Wid, Startupinfo.Hgt);
    gluPerspective(70.0, Startupinfo.Wid/Startupinfo.Hgt, 0.01, 1000.0)
   end;
  end
  else
  begin
   glViewport(0, 0, Startupinfo.Wid, Startupinfo.Hgt);
     glOrtho(0, Startupinfo.Wid, 0, Startupinfo.Hgt, 0.01, 1000);  // Do the perspective calculations. Last value = max clipping depth
  end;
  
  glMatrixMode(GL_MODELVIEW);         // Return to the modelview matrix
  glLoadIdentity();                   // Reset View
end;


end.
 