unit Octtree;

interface

uses windows,OpenGL12, Globals, TGATexture, Classes, Log;

Const
     TOP_LEFT_FRONT =    0;
     TOP_LEFT_BACK =     1;
     TOP_RIGHT_BACK =    2;
     TOP_RIGHT_FRONT =   3;
     BOTTOM_LEFT_FRONT = 4;
     BOTTOM_LEFT_BACK =  5;
     BOTTOM_RIGHT_BACK = 6;
     BOTTOM_RIGHT_FRONT =7;

type
    poctNode = ^octNode;
    PVector = ^Vector;
    Vector = array[0..2] of GLFloat;


    octList = object
               N: array of poctNode;
               NCount: integer;
               Procedure Clear;
              end;

    octNode = object
        P: Vector;
        Wid, hW: glFloat;

        N: array[0..7] of poctNode;
        HasSubnodes: boolean;
        Fcount: integer;
        F: array of integer;
        Vis: boolean;
        DL: gluint;

  {      Procedure Offset(X,Y,Z: single);
        Procedure Init(X,Y,Z,W: single);
        Function Inside(Q:Vector): integer; //Returns -1 if not inside
        Procedure Split(ind: integer);
        Procedure Destroy;
        Procedure AddtoList(Var L: octList);
   }    end;

    Coord = array[0..1] of GLFloat;
    Lightmap  = array[0..49151] of byte;
    VecList = array of Vector;

    octFileNode = record
           Ind: integer;
           P: Vector;
           Wid: glFloat;
           N: array[0..7] of Integer;
           Fcount: integer;
           F: array of integer;

          end;

    CamPoint = record
                // 0 = Position, 1 = Viewing Target
                P: array[0..1] of Vector; //Position
                C: array[0..1] of Vector; //Control Point
               end;
    CamLine = record
               L: array[0..1] of CamPoint; //Points A and B
               Tim, FadeinTime, FadeoutTime: integer; // in Milliseconds
               Link, Fadein, Fadeout: boolean;
              end;
    FaceData = record
               V, N: array[0..2] of Integer;
               T, L: array[0..2] of Coord;
               Tid, Lid: integer;
               UScroll, VScroll: glFloat;
               Env, Trans, Water: boolean;
               end;
    FaceScroll = record
                   Uf, Vf: glFloat;
                 end;

    POTLFile = ^OTLFile;
    OTLFile = record
               NumO, NumF, NumV, NumL: integer;
               Oc: array of integer;
               O: array of octFileNode;
               F: array of FaceData;
               V: array of Vector;
               L: array of Lightmap;
               Sc: array of FaceScroll;
              end;

    HYBDHeader = record
                 O,F,V,L,T,C: integer;
                 end;
    HYBDFile = record
               Header: HYBDHeader;
                O: array of octFileNode;
                F: array of FaceData;
                V: array of Vector;
                L: array of Lightmap;
                T: array of String[40]; //Texture Names;
                C: array of CamLine;
                Tex: array of gluint;
                Sc: array of Facescroll;
                LmTex: array of gluint;
                DL: gluint;
                Oct: POctNode;
                Vis: array of array of boolean;

               end;



var octMaxNodeLevels: integer = 11;
    octMaxNodeFaceCount: integer = 1000;

//Function makeVector(X,Y,Z: glFloat): Vector;
Function Distance(A,B:Vector): glFloat;
function LoadHybd(Filename: string; var H: HybdFile): boolean;
Procedure LoadHybdTextures(var H: HybdFile);
Procedure UnLoadHybdTextures(var H: HybdFile);

implementation

function CreateTexture(Width, Height, Format : Word; pData : Pointer) : Integer;
var
  Texture : GLuint;
begin
  glGenTextures(1, Texture);
  glBindTexture(GL_TEXTURE_2D, Texture);
  //glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);  {Texture blends with object background}

  { Select a filtering type. BiLinear filtering produces very good results with little performance impact
    GL_NEAREST               - Basic texture (grainy looking texture)
    GL_LINEAR                - BiLinear filtering
    GL_LINEAR_MIPMAP_NEAREST - Basic mipmapped texture
    GL_LINEAR_MIPMAP_LINEAR  - BiLinear Mipmapped texture
  }

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); { only first two can be used }
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); { all of the above can be used }

  if Format = GL_RGBA then
    gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, Width, Height, GL_RGBA, GL_UNSIGNED_BYTE, pData)
  else
    gluBuild2DMipmaps(GL_TEXTURE_2D, 3, Width, Height, GL_RGB, GL_UNSIGNED_BYTE, pData);

  result :=Texture;
end;


Procedure CreateLMTexture(var H: hybdfile; pData: pointer; ind: integer);
begin
  glEnable(GL_TEXTURE_2D);
  H.LmTex[Ind] := CreateTexture(128, 128, GL_RGB, pData);
  glBindTexture(GL_TEXTURE_2D, H.LmTex[Ind]);
end;


Procedure CopyOct(A: poctNode; B: octfilenode);
Var I: integer;
begin
   A^.P := B.P;
   A^.Wid := B.Wid;
   A^.hW := A^.Wid /2;
   A^.Fcount := B.Fcount;
   SetLength(A^.F, high(b.f)+1);
   A^.Fcount := High(A^.f);
   For I := 0 to A^.Fcount do
    A^.F[I] := B.F[I];
   A^.HasSubnodes := False;
   For I := 0 to 7 do
   if B.N[I] <> -1 then
   A^.HasSubnodes := true;
end;



Procedure FillOct(A: POctNode;H: hybdFile; Ind: integer);
Var I,J: integer;
    B: OctFileNode;
    Fnd: boolean;
begin

  Addtolog('Parsing OCT index ' + inttostr(ind));
  Fnd := False;
  For J := 0 to high(H.O) do
  if H.O[J].Ind = ind then
  begin
  B := H.O[J];
  Fnd := True;
  end;

  if Fnd = False then exit;
  CopyOct(A, B);
  For I := 0 to 7 do
  A^.N[I] := nil;

  if A.HasSubnodes then
  begin
    Addtolog('Has Children - Populating');
    For I := 0 to 7 do
    if B.N[I] <> -1 then
    begin
     New(A^.N[I]);
     //FillOct(A.N[I],H, B.N[I]);
    end;
  end;


  if A.HasSubnodes then
  For I := 0 to 7 do
  if A.N[I] <> nil then
    if B.N[I] <> -1 then
     FillOct(A.N[I], H, B.N[I]);

end;


function LoadHybd(Filename: string; var H: HybdFile): boolean;
Var F: TFilestream;
    I, J: integer;
    F1, F2: string;
    Done: boolean;
    Fl: file of byte;
begin
   Addtolog('Loading HYBD: '+ filename);

   Result := true;
   Done := False;
  try
  try
   F := TFilestream.Create(filename, fmOpenRead);
   F.ReadBuffer(H.Header, sizeof(HybdHeader));

   Setlength(H.O, H.Header.O+1);
   Setlength(H.F, H.Header.F+1);
   Setlength(H.V, H.Header.V+1);
   Setlength(H.L, H.Header.L+1);
   Setlength(H.T, H.Header.T+1);
   Setlength(H.C, H.Header.C+1);

   For I := 0 to H.Header.O do
   begin
      f.ReadBuffer(h.O[I].Ind, sizeof(h.O[I].Ind));
      f.ReadBuffer(h.O[I].P, sizeof(h.O[I].P));
      f.ReadBuffer(h.O[I].Wid, sizeof(h.O[I].Wid));
      f.ReadBuffer(h.O[I].N, sizeof(h.O[I].N));
      f.ReadBuffer(h.O[I].Fcount, sizeof(h.O[I].Fcount));

      Setlength(H.O[I].F, H.O[I].Fcount+1);
      for J := 0 to H.o[I].Fcount do
      f.ReadBuffer(H.O[I].F[J], Sizeof(H.O[I].F[J]));
   end;
   For I := 0 to H.Header.F do
     f.REadBuffer(H.F[I], Sizeof(FaceData));
   For I := 0 to H.Header.V do
     f.ReadBuffer(H.V[I], Sizeof(Vector));
   For I := 0 to H.Header.L do
     f.readBuffer(H.L[I], Sizeof(Lightmap));
   For I := 0 to H.Header.T do
   begin
     f.ReadBuffer(H.T[I], Sizeof(H.T[I]));
   end;
   For I := 0 to H.Header.C do
     f.ReadBuffer(H.C[I], Sizeof(CamLine));

   Setlength(H.Vis, H.Header.F+1);
   For I := 0 to H.Header.F do
   Setlength(H.Vis[I], H.Header.C+1);

   for I := 0 to H.Header.F do
   for J := 0 to H.Header.C do
    F.ReadBuffer(H.Vis[I][J], sizeof(boolean));

   Done := true;
  Finally

   if not done then
   begin
    Result := false;
    addtolog('Hybd NOT loaded correctly.')
   end
   else
   Addtolog('Hybd loaded OK.');

   F.Free;
  end;
  except
   Result := false;
   Addtolog('An exception occured in Hybd Load procedure.');
  end;

  //Addtolog('Building OCTNodes...');
  //New(H.Oct);
  //FillOct(H.Oct, H, 0);

  Addtolog('Destroying OctFileNode structure.');
  SEtlength(H.O, 0);



  SetLength(H.Sc, High(H.F)+1);
  Setlength(H.Lmtex, High(H.L)+1);
  For I := 0 to high(H.L) do
  begin
   Addtolog('Extracting Lightmap ' + inttostr(I));
   CreateLMTexture(H, @H.L[I], I);
  end;
end;

Procedure LoadHybdTextures(var H: HybdFile);
Var I: integer;
begin
   setlength(H.Tex, High(H.T)+1);
   For I := 0 to high(H.T) do
   begin
      LoadJPG(TexturePath+H.T[I], H.Tex[I]);
   end;


end;

Procedure UnLoadHybdTextures(var H: HybdFile);
Var I: integer;
begin
  if GL_VERSION_1_2 then
  begin
   For I := 0 to high(H.T) do
   begin
      glDeleteTextures(1, @H.Tex[I]);
   end;

   Setlength(H.Tex, 0);
  end;
end;



Function Distance(A,B:Vector): glFloat;
Var Dx, Dy, Dz: glfloat;
begin
    Dx := abs(A[0]-B[0]);
    Dy := abs(A[1]-B[1]);
    Dz := abs(A[2]-B[2]);

    Result := sqrt(Dx*Dx+Dy*DY+Dz*Dz);
end;

Procedure octList.Clear;
begin
    SetLength(N, 0);
    NCount := 0;
end;

{Procedure octNode.AddtoList(Var L: octList);
Var I: integer;
begin
  SetLength(l.N, high(l.n)+2);
  L.N[high(l.n)] := @self;
  l.Ncount := high(l.n);

  for I := 0 to 7 do
  if N[I] <> nil then
  begin
     N[I].Addtolist(L);
  end;
end;

Procedure octNode.Offset(X,Y,Z: single);
Var I: integer;
begin
    P[0] := P[0]+X;
    P[1] := P[1]+Y;
    P[2] := P[2]+Z;

    For I := 0 to 7 do
    if N[I] <> nil then
    begin
       N[I].Offset(X,Y,Z);
    end;
end;

procedure octNode.Destroy;
Var I: integer;
begin
   For I := 0 to 7 do
   if N[I] <> nil then
   begin
    N[I].Destroy;
//    Dispose(N[I]);
   end;

   For I := 0 to 7 do
   if N[I] <> nil then
   begin
    Dispose(N[I]);
   end;


   SetLength(F, 0);
   Hassubnodes := False;
end;

Function makeVector(X,Y,Z: glFloat): octtree.Vector;
begin
   Result[0]:=X;
   Result[1]:=Y;
   Result[2]:=Z;
end;

Procedure octNode.Init(X,Y,Z,W: single);
Var I: integer;
begin
    P[0] := X;
    P[1] := Y;
    P[2] := Z;
    Wid := W;
    hW := Wid/2;

    for I := 0 to 7 do
    N[I] := Nil;

    HasSubnodes := false;
    FCount := 0;
    SetLength(F, 0);
end;

Procedure octNode.Split(ind: integer);
Var X,Y,Z: glFloat;
begin
   if Ind = -1 then exit;
   if N[ind] <> nil then Exit;
   New(N[ind]);

   X:=0;
   Y:=0;
   Z:=0;

   Case ind of
   TOP_LEFT_FRONT: begin
      X := P[0]-hw/2;
      Y := P[1]+hw/2;
      Z := P[2]+hw/2;
      end;
   TOP_LEFT_BACK: begin
      X := P[0]-hw/2;
      Y := P[1]+hw/2;
      Z := P[2]-hw/2;
      end;
   TOP_RIGHT_BACK: begin
      X := P[0]+hw/2;
      Y := P[1]+hw/2;
      Z := P[2]-hw/2;
      end;
   TOP_RIGHT_FRONT: begin
      X := P[0]+hw/2;
      Y := P[1]+hw/2;
      Z := P[2]+hw/2;
      end;
   BOTTOM_LEFT_FRONT: begin
      X := P[0]-hw/2;
      Y := P[1]-hw/2;
      Z := P[2]+hw/2;
      end;
   BOTTOM_LEFT_BACK: begin
      X := P[0]-hw/2;
      Y := P[1]-hw/2;
      Z := P[2]-hw/2;
      end;
   BOTTOM_RIGHT_BACK: begin
      X := P[0]+hw/2;
      Y := P[1]-hw/2;
      Z := P[2]-hw/2;
      end;
   BOTTOM_RIGHT_FRONT: begin
      X := P[0]+hw/2;
      Y := P[1]-hw/2;
      Z := P[2]+hw/2;
      end;
   end;
   fCount := 0;
   Setlength(F, 0);

   N[ind].Init(X,Y,Z, hw);

   HasSubnodes := True;
end;


function Inn(X, X1, X2: glFloat): boolean;
begin
   Result := (X >= X1) and (X <= X2);
end;

Function octNode.Inside(Q:Vector): integer;
//Var qw: glFloat;
begin
  Result := -1;

  if Inn(Q[0], P[0]-Hw, P[0]+Hw) then
  if Inn(Q[1], P[1]-Hw, P[1]+Hw) then
  if Inn(Q[2], P[2]-Hw, P[2]+Hw) then
  begin

  if (Q[0] <= P[0]) and (Q[1] >= P[1]) and (Q[2] >= P[2]) then
   Result := 0;    //TOP_LEFT_FRONT
  if (Q[0] <= P[0]) and (Q[1] >= P[1]) and (Q[2] <= P[2]) then
   Result := 1;    //TOP_LEFT_BACK
  if (Q[0] >= P[0]) and (Q[1] >= P[1]) and (Q[2] <= P[2]) then
   Result := 2;    //TOP_RIGHT_BACK
  if (Q[0] >= P[0]) and (Q[1] >= P[1]) and (Q[2] >= P[2]) then
   Result := 3;    //TOP_RIGHT_FRONT
  if (Q[0] <= P[0]) and (Q[1] <= P[1]) and (Q[2] >= P[2]) then
   Result := 4;    //BOTTOM_LEFT_FRONT
  if (Q[0] <= P[0]) and (Q[1] <= P[1]) and (Q[2] <= P[2]) then
   Result := 5;    //BOTTOM_LEFT_BACK
  if (Q[0] >= P[0]) and (Q[1] <= P[1]) and (Q[2] <= P[2]) then
   Result := 6;    //BOTTOM_RIGHT_BACK
  if (Q[0] >= P[0]) and (Q[1] <= P[1]) and (Q[2] >= P[2]) then
   Result := 7;    //BOTTOM_RIGHT_FRONT

  end;

   {  //qw := Hw/2;
     if Q[1] > P[1] then //Top
     begin
       if Q[2] < P[2] then //Closest
       begin
         if Q[0] < P[0] then //Left
         begin
           Result := 0;
           exit;
         end                    //Right
         else
         begin
           Result := 1;
           exit;
         end;
       end
       else     // Fartherest
       begin
         if Q[0] < P[0] then //Left
         begin
           Result := 2;
           exit;
         end
         else                  //Right
         begin
           Result := 3;
           exit;
         end;
       end;
     end
     else      //BOTTOM
     begin
       if Q[2] < P[2] then //Closest
       begin
         if Q[0] < P[0] then //Left
         begin
           Result := 4;
           exit;
         end                    //Right
         else
         begin
           Result := 5;
           exit;
         end;
       end
       else     // Fartherest
       begin
         if Q[0] < P[0] then //Left
         begin
           Result := 6;
           exit;
         end
         else                  //Right
         begin
           Result := 7;
           exit;
         end;
       end;
     end;
    }
{end;}

end.
