{-----------------------------------------------------------------------------
 Unit Name: Water
 Author:    Nitrogen
 Purpose:   Water layer..
 History:
-----------------------------------------------------------------------------}


unit Water;

interface

Uses windows, Opengl12, Globals;


Procedure Water_Init;
Procedure Water_Draw;
procedure Water_AddDrop;


implementation

Uses TGATexture;

const
  GridSize = 63;

type TGLCoord = Record
       X, Y, Z : glFloat;
     end;


Var
  WaterTexture : glUint;
  Viscosity : glFloat;
  Position : Array[0..GridSize, 0..GridSize] of glFloat;
  Velocity : Array[0..GridSize, 0..GridSize] of glFloat;

  Vertex : Array[0..GridSize, 0..GridSize] of TglCoord;
  Normals:array [0..GridSize, 0..GridSize] of TglCoord;


procedure Water_adddrop;
begin
  Velocity[random(GridSize-3)+2, random(GridSize-3)+2] :=1560;
end;


Procedure Water_Init;
Var I, J: integer;
begin

  LoadJPG('Data\Reflection.jpg', WaterTexture);

  glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
  glEnable(GL_LIGHT0);

  Viscosity :=0.96;
  For I :=0 to GridSize do
  begin
    For J :=0 to GridSize do
    begin
      Position[I, J] :=0;
      Velocity[I, J] :=0;
    end;
  end;


end;

procedure Water_Draw;
var I, J : Integer;
    VectLength : glFloat;
begin


  // Calculate new velocity
  For I :=2 to GridSize-2 do
    For J :=2 to GridSize-2 do
      Velocity[I, J] := Velocity[I, J] + (Position[I, J] -
              (4*(Position[I-1,J] + Position[I+1,J] + Position[I,J-1] + Position[I,J+1]) +  // left, right, above, below
              Position[I-1,J-1] + Position[I+1,J-1] + Position[I-1,J+1] + Position[I+1,J+1])/25) / 7;  // diagonally across

  // Calculate the new ripple positions
  For I:=2 to GridSize-2 do
    For J:=2 to GridSize-2 do
    Begin
      Position[I, J] := Position[I, J] - Velocity[I,J];
      Velocity[I, J] := Velocity[I, J] * Viscosity;
    End;

  // Calculate the new vertex coordinates
  For I :=0 to GridSize do
    For J :=0 to GridSize do
    begin
      Vertex[I, J].X :=(I - GridSize/2)/GridSize*15;
      Vertex[I, J].z :=(Position[I, J] / 1024)/GridSize*6;
      Vertex[I, J].y :=(J - GridSize/2)/GridSize*15;
    end;

  // Calculate the new vertex normals.
  // Do this by using the points to each side to get the right angle
  For I :=0 to GridSize do
  begin
    For J :=0 to GridSize do
    begin
      If (I > 0) and (J > 0) and (I < GridSize) and (J < GridSize) then
      begin
        with Normals[I, J] do
        begin
          X := Position[I+1, J] - Position[I-1,J];
          Y := -2048;
          Z := Position[I, J+1] - Position[I, J-1];

          VectLength :=sqrt(x*x + y*y + z*z);
          if VectLength <> 0 then
          begin
            X :=X/VectLength;
            Y :=Y/VectLength;
            Z :=Z/VectLength;
          end;
        end;
      end
      else
      begin
        Normals[I, J].X :=0;
        Normals[I, J].Y :=1;
        Normals[I, J].Z :=0;
      end;
    end;
  end;

  // Draw the water texture
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glDisable(GL_TEXTURE_2D);


 glActiveTextureARB(GL_TEXTURE0_ARB);
  glBindTexture(GL_TEXTURE_2D, WaterTexture);
 glEnable(GL_BLEND);

  glEnable(GL_TEXTURE_GEN_S);
  glEnable(GL_TEXTURE_GEN_T);
 glDisable(GL_LIGHTING);
 glColor3f(0.5,0.5,0.5);
  For J :=0 to GridSize-1 do
  begin
    glBegin(GL_QUAD_STRIP);
      for I :=0 to GridSize do
      begin
        glNormal3fv(@Normals[I, J+1]);
//        glTexCoord2f(I/GridSize, (J+1)/GridSize);
        glVertex3fv(@Vertex[I, J+1]);
        glNormal3fv(@Normals[I, J]);
//        glTexCoord2f(I/GridSize, J/GridSize);
        glVertex3fv(@Vertex[I, J]);
      end;
    glEnd;
  end;
 glColor3f(1,1,1);
  glDisable(GL_LIGHTING);
  glDisable(GL_TEXTURE_GEN_S);
  glDisable(GL_TEXTURE_GEN_T);
 glDisable(GL_BLEND);


 glActiveTextureARB(GL_TEXTURE1_ARB);
 glEnable(GL_TEXTURE_2D);
 glActiveTextureARB(GL_TEXTURE0_ARB);


end;



end.
